/***************************************************************************
                          bcdoc.h  -  description
                             -------------------
    begin                : Sun May 27 22:13:58 CEST 2001
    copyright            : (C) 2001 by Michael Andreen
    email                : whale@linux.nu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef BCDOC_H
#define BCDOC_H

// include files for QT
#include <qobject.h>

// standard library
#include <map>

// application specific includes
#include "fleet.h"

typedef std::map<QString, std::map<QString, std::map<QString, Fleet*> > > BattleList;
//! The document class
/** This class is holding all the data, handles the transportation of the
  * data to a couple of different mediums and informs users of the data
  * if it's changed (through the documentChnaged() signal)
  * \bug it's too open.. 
  * \todo make this class more closed.
  * \author Michael Andreen
  */

class BSDoc : public QObject
{
  Q_OBJECT

  public:
	BSDoc();
    ~BSDoc();
    void newDoc();
    bool save();
    bool saveAs(const QString &filename);
    bool load(const QString &filename);
    bool isModified() const;

	/**Adds a new battle to the dokument and emits documentChanged.
	 * \todo add some checks to see if the battle exist and so and return the status.
	 * \return returns 0 if everything is ok.
	 */
	int newBattle(QString name);

	/**This returns the the data structure, so other can work on it, but not supposed
	 * change it.
	 * \todo remove this, and replace it with a better more closed interface.
	 */
	const BattleList& battles() const;

	
	/**Looks for the fleet with the specified name, in the specified group in the
	 * specified battle.
	 * \return Returns '\0' (NULL) if the fleet isn't found, if it is found it
	 * returns a const pointer to the fleet.
	 */
	const Fleet* specificFleet(QString battle, QString group, QString fleet) const;

	void changeFleet(QString battle, QString group, QString fleet, const Fleet* fl);

	void newFleet(QString battle, QString group, QString fleet, Fleet* fl);

	void removeFleet(QString battle, QString group, QString fleet);

	void runBattleSimulation();
  signals:
    void documentChanged();

  protected:
    bool modified;

	/**This is the main datastructure of the battlecalc, it's more or less the
	 * core that everything else works around. A short explanation of the fields:
	 * -# The name of the battle (usually coordinates)
	 * -# The name of the group (ie. Friendly/Hostile)
	 * -# The fleetname (Home Planet, coordinates, irc nicks and so on)
	 * - and in the end you got the actual Fleet object with it's data.
	 */
	 BattleList m_Battles;
};

#endif
